import {
  select,
  selectAll,
  scaleOrdinal,
  schemeCategory10,
  schemePastel1 
} from 'd3';

import { colorLegend } from './colorLegend'; 
 
const svg = select('svg');  

const width = +svg.attr('width'); 
const height = +svg.attr('height'); 

const margin = { top: 60, right: 160, bottom: 88, left: 120 }; 
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;

const jsonNodeValue = 'group'
const nodeColorValue = d => d[jsonNodeValue];

const jsonLinkValue = 'dependency'
const linkColorValue = d => d[jsonLinkValue];

const g = svg.append('g')
  .attr('transform', `translate(${margin.left},${margin.top})`);

const circleColorScale = scaleOrdinal(schemeCategory10);
const lineColorScale = scaleOrdinal(schemePastel1);

let selectedColorValue; 



const render = data => {
  var simulation = d3.forceSimulation(data.nodes)
    .force("link", d3.forceLink(data.links))
    .force('charge', d3.forceManyBody().strength(-200))
    .force('center', d3.forceCenter(innerWidth / 2, innerHeight / 2))
    .on('tick', ticked);
  
  var link = g.selectAll('line')
  	.data(data.links)
  	.enter()
  	.append('line')
  	.attr("class", "link")
  	.attr("stroke-width", 1)
  	.style('stroke',  d => lineColorScale(d[jsonLinkValue]));
  
  var nodes = g.append("g")
  	.selectAll("g")
  	.data(data.nodes)
  	.enter()
  	.append("g")
  	.attr("class", "node")
  	.call(d3.drag()
      .on("start", dragstarted)
      .on("drag", dragged)
      .on("end", () => {
      		dragended;
          nodes.remove();      		
          link.remove();      		
          render(data);
          }));
  
  var circles = nodes
  	.append("circle")
    .attr("r", 2)
    .attr('pointer-events', d => colorStatementSelector(circleColorScale(d[jsonNodeValue]),'auto', 'none'))
    .attr('opacity',  d =>
      (circleColorScale(d[jsonNodeValue]) === selectedColorValue || !selectedColorValue)
        ? 1
        : 0.2
      )
  	.attr("fill", d => circleColorScale(d[jsonNodeValue]));
  
  var texts = nodes
  	.append("text")
  	.text(d => d.name)
  	.attr("font-size", 5);
  
  function ticked() {
    nodes
      .attr("transform", d => "translate(" + d.x + ", " + d.y + ")")

    link
      .attr('x1', d => d.source.x)
      .attr('y1', d => d.source.y)
      .attr('x2', d => d.target.x)
      .attr('y2', d => d.target.y);
  }
  
  function dragstarted(d){
    simulation.alphaTarget(0.3).restart();
    d.fx = d3.event.x;
    d.fy = d3.event.y;
  }

  function dragged(d){
    d.fx = d3.event.x;
    d.fy = d3.event.y;
  }  

  function dragended(d){
    simulation.alphaTarget(0);
    d.fx = null;
    d.fy = null;
  }

}





function colorStatementSelector(color, trueStatement, falseStatement) {
  var returnValue;
  (color === selectedColorValue || !selectedColorValue)
      ? returnValue = trueStatement
      : returnValue = falseStatement
  return returnValue;
}

const onClick = d => {
  console.log(d)
  if (d !== null){
    if (circleColorScale.domain().includes(d))
      selectedColorValue = circleColorScale(d);
    if (lineColorScale.domain().includes(d))
      selectedColorValue = null;
  }
  else
    selectedColorValue = null
};

d3.json('data.json')
  .then(data => {
  
  console.log(data.links)
  
  circleColorScale
  	.domain(data.nodes.map(nodeColorValue));
  
  console.log(circleColorScale.domain())
  
  lineColorScale
  	.domain(data.links.map(linkColorValue).sort());
  

  
  
  
  
//   const nested = nest()
//   	.key(xValue);
  
// 	var dataByGroup = nested.entries(data);
  
   render(data);
  
//   g.append('text')
//     .attr('class', 'title')
//     .attr('y', -10)
//     .attr('x', innerWidth / 2)
//     .attr('text-anchor', 'middle')
//     .text(title);
  
  svg.append('g')
    .attr('class', 'circle legend')
    .attr('transform', `translate(830,121)`)
    .call(colorLegend, {
      colorScale: circleColorScale,
      circleRadius: 5,
      spacing: 15,
      textOffset: 10,
    	onClick
    });
  
  svg.append('g')
    .attr('class', 'line legend')
    .attr('transform', `translate(830,400)`)
    .call(colorLegend, {
      colorScale: lineColorScale,
      circleRadius: 5,
      spacing: 15,
      textOffset: 10,
    	onClick
    });
  });



